using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace Sudoku.Controls
{
	#region delegates
	public delegate void tbFilledEventHandler(object sender, KeyDownEventArgs e);
	public delegate void tbClearedEventHandler(object sender, KeyDownEventArgs e);
	#endregion

	#region classes

	#region class TextButton
	/// <summary>
	/// Summary description for TextButton.
	/// </summary>
	internal class TextButton : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		internal TextButton()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{

		}
		#endregion

		#region events
		public event tbFilledEventHandler Filled;
		public event tbClearedEventHandler Cleared;
		#endregion

		#region private variables
		private Numbers Nos = new Numbers();
		private string _Number = String.Empty;
		private byte _Row = 0;
		private byte _Column = 0;
		#endregion

		#region properties

		public bool HasNumber
		{
			get {return Number!=String.Empty;}
		}
		public bool NumberIsActive(string Number)
		{
			return Nos.ItemByValue(Convert.ToInt32(Number)).Active;
		}

		public string Number
		{
			get { return _Number; }
			set 
			{ 
				_Number = value; 
				DrawTextButton();
			}
		}	

		public byte Row
		{
			get { return _Row; }
			set { _Row = value; }
		}

		public byte Column
		{
			get { return _Column; }
			set { _Column = value; }
		}


		#endregion

		#region overridden methods
		protected override void OnPaint(PaintEventArgs e)
		{
			ControlPaint.DrawBorder3D(this.CreateGraphics(), new Rectangle(0,0,this.Width,this.Height), Border3DStyle.Raised, Border3DSide.All);
			if (HasNumber) DrawTextButton();
			base.OnPaint (e);
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			ControlPaint.DrawBorder3D(this.CreateGraphics(), new Rectangle(0,0,this.Width,this.Height), Border3DStyle.Sunken, Border3DSide.All);
			if (HasNumber) DrawTextButton();
			base.OnMouseDown (e);
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			ControlPaint.DrawBorder3D(this.CreateGraphics(), new Rectangle(0,0,this.Width,this.Height), Border3DStyle.Raised, Border3DSide.All);
			if (HasNumber) DrawTextButton();
			base.OnMouseLeave (e);
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			base.OnKeyDown (e);
			if ((e.KeyCode >= Keys.NumPad1 & e.KeyCode <= Keys.NumPad9) | (e.KeyCode >= Keys.D1 & e.KeyCode <= Keys.D9))
			{
				string newNumber = e.KeyCode.ToString().Substring(e.KeyCode.ToString().Length - 1, 1);
				// Werd er een andere nummer gekozen ?
				if (newNumber!=Number)
				{
					// Kan de nummer worden gekozen ?
					if (NumberIsActive(newNumber)) 
					{
						// event
						KeyDownEventArgs kea = new KeyDownEventArgs();
						kea.NewNumber=newNumber;
						Filled(this, kea);
						if (!kea.Invalid) 
						{
							// oude nummer mag terug worden gekozen
							if (HasNumber) IncludeNumber(Number);
							// nummer mag niet meer worden gekozen
							ExcludeNumber(newNumber);
							// huidige nummer bijhouden
							Number = newNumber;
						}
					}
				}
			}
			if (e.KeyCode == Keys.Back | e.KeyCode == Keys.Delete)
			{
				if (HasNumber)
				{
					IncludeNumber(Number);
					Cleared(this, new KeyDownEventArgs());
					Number = String.Empty;
				}
			}		
			else
				e.Handled = true;
		}
		#endregion

		#region private methods
		private void DrawTextButton()
		{
			if (HasNumber)
			{
				//Teken border van control
				ControlPaint.DrawBorder3D(this.CreateGraphics(), new Rectangle(0,0,this.Width,this.Height), Border3DStyle.RaisedInner, Border3DSide.All);
				//Teken de tekst in het midden
				float tX = 0;
				float tY = 0;
				
				Font fnt = new Font("Arial", 16, FontStyle.Bold);
				SizeF tS = this.CreateGraphics().MeasureString(Number, fnt, System.Convert.ToInt32(this.Width));

				tX = (this.Width - tS.Width)  / 2;
				tY = (this.Height - tS.Height) / 2;
				StringFormat sF = new StringFormat();
				sF.Alignment = StringAlignment.Center;	
				this.CreateGraphics().DrawString(Number, fnt, new SolidBrush(System.Drawing.Color.Black), new RectangleF(tX, tY, tS.Width, tS.Height), sF);
			}
			else
			{
				ControlPaint.DrawBorder3D(this.CreateGraphics(), new Rectangle(0,0,this.Width,this.Height), Border3DStyle.Raised, Border3DSide.All);
				this.Focus();
			}
		}

		private void ExcludeNumber(string Number)
		{
			Nos.ItemByValue(Convert.ToInt32(Number)).Active=false;
		}

		private void IncludeNumber(string Number)
		{
			Nos.ItemByValue(Convert.ToInt32(Number)).Active=true;
		}

		#endregion

		#region event - handling

		#endregion
	}
	#endregion

	#region class KeyDownEventArgs
	public class KeyDownEventArgs : EventArgs
	{
		private bool _Invalid;
		private string _newNumber;

		public bool Invalid
		{
			get { return _Invalid; }
			set { _Invalid = value; }
		}

		public string NewNumber
		{
			get { return _newNumber; }
			set { _newNumber = value; }
		}

	}
	#endregion

	#endregion

}
